@@ -169,7 +169,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         return volume_paths
 
     @utils.trace
-    @synchronized('extend_volume')
+    @synchronized('extend_volume', external=True)
     def extend_volume(self, connection_properties):
         """Update the local kernel's size information.
 
@@ -190,7 +190,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             raise exception.VolumePathsNotFound()
 
     @utils.trace
-    @synchronized('connect_volume')
+    @synchronized('connect_volume', external=True)
     def connect_volume(self, connection_properties):
         """Attach the volume to instance_name.
 
@@ -225,8 +225,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
                           {'device': device})
                 if os.path.exists(device) and self.check_valid_device(device):
                     self.host_device = device
-                    # get the /dev/sdX device.  This is used
-                    # to find the multipath device.
+                    # get the /dev/sdX device.  This variable is maintained to
+                    # keep the same log output.
                     self.device_name = os.path.realpath(device)
                     raise loopingcall.LoopingCallDone()
 
@@ -260,8 +260,11 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         # see if the new drive is part of a multipath
         # device.  If so, we'll use the multipath device.
         if self.use_multipath:
+            # Pass a symlink, not a real path, otherwise we'll get a real path
+            # back if we don't find a multipath and we'll return that to the
+            # caller, breaking Nova's encryption which requires a symlink.
             (device_path, multipath_id) = self._discover_mpath_device(
-                device_wwn, connection_properties, self.device_name)
+                device_wwn, connection_properties, self.host_device)
             if multipath_id:
                 # only set the multipath_id if we found one
                 device_info['multipath_id'] = multipath_id
@@ -275,12 +278,13 @@ class FibreChannelConnector(base.BaseLinuxConnector):
     def _get_host_devices(self, possible_devs):
         """Compute the device paths on the system with an id, wwn, and lun
 
-        :param possible_devs: list of (pci_id, wwn, lun) tuples
+        :param possible_devs: list of (platform, pci_id, wwn, lun) tuples
         :return: list of device paths on the system based on the possible_devs
         """
         host_devices = []
-        for pci_num, target_wwn, lun in possible_devs:
-            host_device = "/dev/disk/by-path/pci-%s-fc-%s-lun-%s" % (
+        for platform, pci_num, target_wwn, lun in possible_devs:
+            host_device = "/dev/disk/by-path/%spci-%s-fc-%s-lun-%s" % (
+                platform + '-' if platform else '',
                 pci_num,
                 target_wwn,
                 self._linuxscsi.process_lun_id(lun))
@@ -293,7 +297,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         :param hbas: available hba devices.
         :param targets: tuple of possible wwn addresses and lun combinations.
 
-        :returns: list of (pci_id, wwn, lun) tuples
+        :returns: list of (platform, pci_id, wwn, lun) tuples
 
         Given one or more wwn (mac addresses for fibre channel) ports
         do the matrix math to figure out a set of pci device, wwn
@@ -303,15 +307,15 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         """
         raw_devices = []
         for hba in hbas:
-            pci_num = self._get_pci_num(hba)
+            platform, pci_num = self._get_pci_num(hba)
             if pci_num is not None:
                 for wwn, lun in targets:
                     target_wwn = "0x%s" % wwn.lower()
-                    raw_devices.append((pci_num, target_wwn, lun))
+                    raw_devices.append((platform, pci_num, target_wwn, lun))
         return raw_devices
 
     @utils.trace
-    @synchronized('connect_volume')
+    @synchronized('connect_volume', external=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
         """Detach the volume from instance_name.
@@ -343,8 +347,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
                 mpath_path = self._linuxscsi.find_multipath_device_path(wwn)
                 if mpath_path:
                     self._linuxscsi.flush_multipath_device(mpath_path)
-            device_info = self._linuxscsi.get_device_info(real_path)
-            devices.append(device_info)
+            dev_info = self._linuxscsi.get_device_info(real_path)
+            devices.append(dev_info)
 
         LOG.debug("devices to remove = %s", devices)
         self._remove_devices(connection_properties, devices, device_info)
@@ -355,7 +359,15 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         # all of them
         path_used = self._linuxscsi.get_dev_path(connection_properties,
                                                  device_info)
-        was_multipath = '/pci-' not in path_used
+        # NOTE: Due to bug #1897787 device_info may have a real path for some
+        # single paths instead of a symlink as it should have, so it'll only
+        # be a multipath if it was a symlink (not real path) and it wasn't a
+        # single path symlink (those have filenames starting with pci-)
+        # We don't use os.path.islink in case the file is no longer there.
+        was_symlink = path_used.count(os.sep) > 2
+        # We check for /pci because that's the value we return for single
+        # paths, whereas for multipaths we have multiple link formats.
+        was_multipath = '/pci-' not in path_used and was_symlink
         for device in devices:
             device_path = device['device']
             flush = self._linuxscsi.requires_flush(device_path,
@@ -370,10 +382,18 @@ class FibreChannelConnector(base.BaseLinuxConnector):
         # /sys/devices/pci0000:20/0000:20:03.0/0000:21:00.2/net/ens2f2/ctlr_2
         # /host3/fc_host/host3
         # we always want the value prior to the host or net value
+        # on non x86_64 device, pci devices may be appended on platform device,
+        # /sys/devices/platform/smb/smb:motherboard/80040000000.peu0-c0/pci0000:00/0000:00:03.0/0000:05:00.3/host2/fc_host/host2  # noqa
+        # so also return a platform id if it exists
+        platform = None
         if hba is not None:
             if "device_path" in hba:
                 device_path = hba['device_path'].split('/')
+                has_platform = (len(device_path) > 3
+                                and device_path[3] == 'platform')
                 for index, value in enumerate(device_path):
+                    if has_platform and value.startswith('pci'):
+                        platform = "platform-%s" % device_path[index - 1]
                     if value.startswith('net') or value.startswith('host'):
-                        return device_path[index - 1]
-        return None
+                        return platform, device_path[index - 1]
+        return None, None
