@@ -30,6 +30,7 @@ from os_brick import utils
 
 LOG = logging.getLogger(__name__)
 DEVICE_SCAN_ATTEMPTS_DEFAULT = 3
+CONNECTOR_CONF_PATH = '/opt/emc/scaleio/openstack/connector.conf'
 synchronized = lockutils.synchronized_with_prefix('os-brick-')
 
 
@@ -86,6 +87,27 @@ class ScaleIOConnector(base.BaseLinuxConnector):
             LOG.error(msg)
             raise exception.BrickException(message=msg)
 
+    @staticmethod
+    def _get_password_token(connection_properties):
+        # In old connection format we had the password and token in properties
+        if 'serverPassword' in connection_properties:
+            return (connection_properties['serverPassword'],
+                    connection_properties['serverToken'])
+
+        # The new format reads password from file and doesn't have the token
+        LOG.info("Get ScaleIO connector password from configuration file")
+        try:
+            password = priv_scaleio.get_connector_password(
+                CONNECTOR_CONF_PATH,
+                connection_properties['config_group'],
+                connection_properties.get('failed_over', False))
+            return password, None
+        except Exception as e:
+            msg = _("Error getting ScaleIO connector password from "
+                    "configuration file: %s") % e
+            LOG.error(msg)
+            raise exception.BrickException(message=msg)
+
     def _rescan_vols(self):
         LOG.info("ScaleIO rescan volumes")
 
@@ -306,8 +328,8 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         self.server_ip = connection_properties['serverIP']
         self.server_port = connection_properties['serverPort']
         self.server_username = connection_properties['serverUsername']
-        self.server_password = connection_properties['serverPassword']
-        self.server_token = connection_properties['serverToken']
+        self.server_password, self.server_token = self._get_password_token(
+            connection_properties)
         self.iops_limit = connection_properties['iopsLimit']
         self.bandwidth_limit = connection_properties['bandwidthLimit']
         device_info = {'type': 'block',
@@ -315,7 +337,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         return device_info
 
     @utils.trace
-    @lockutils.synchronized('scaleio', 'scaleio-')
+    @lockutils.synchronized('scaleio', 'scaleio-', external=True)
     def connect_volume(self, connection_properties):
         """Connect the volume.
 
@@ -429,7 +451,7 @@ class ScaleIOConnector(base.BaseLinuxConnector):
         return device_info
 
     @utils.trace
-    @lockutils.synchronized('scaleio', 'scaleio-')
+    @lockutils.synchronized('scaleio', 'scaleio-', external=True)
     def disconnect_volume(self, connection_properties, device_info,
                           force=False, ignore_errors=False):
         """Disconnect the ScaleIO volume.
