@@ -119,8 +119,20 @@ class LinuxSCSI(executor.Executor):
 
         return dev_info
 
-    def get_sysfs_wwn(self, device_names):
+    def get_sysfs_wwn(self, device_names, mpath=None):
         """Return the wwid from sysfs in any of devices in udev format."""
+        # If we have a multipath DM we know that it has found the WWN
+        if mpath:
+            # We have the WWN in /uuid even with friendly names, unline /name
+            try:
+                with open('/sys/block/%s/dm/uuid' % mpath) as f:
+                    # Contents are matph-WWN, so get the part we want
+                    wwid = f.read().strip()[6:]
+                    if wwid:  # Check should not be needed, but just in case
+                        return wwid
+            except Exception as exc:
+                LOG.warning('Failed to read the DM uuid: %s', exc)
+
         wwid = self.get_sysfs_wwid(device_names)
         glob_str = '/dev/disk/by-id/scsi-'
         wwn_paths = glob.glob(glob_str + '*')
@@ -128,16 +140,34 @@ class LinuxSCSI(executor.Executor):
         if wwid and glob_str + wwid in wwn_paths:
             return wwid
 
-        # If we have multiple designators follow the symlinks
+        # If we have multiple designators use symlinks to find out the wwn
+        device_names = set(device_names)
         for wwn_path in wwn_paths:
             try:
                 if os.path.islink(wwn_path) and os.stat(wwn_path):
                     path = os.path.realpath(wwn_path)
-                    if path.startswith('/dev/') and path[5:] in device_names:
-                        return wwn_path[len(glob_str):]
+                    if path.startswith('/dev/'):
+                        name = path[5:]
+                        # Symlink may point to the multipath dm if the attach
+                        # was too fast or we took long to check it. Check
+                        # devices belonging to the multipath DM.
+                        if name.startswith('dm-'):
+                            # Get the devices that belong to the DM
+                            slaves_path = '/sys/class/block/%s/slaves' % name
+                            dm_devs = os.listdir(slaves_path)
+                            # This is the right wwn_path if the devices we have
+                            # attached belong to the dm we followed
+                            if device_names.intersection(dm_devs):
+                                break
+
+                        # This is the right wwn_path if  devices we have
+                        elif name in device_names:
+                            break
             except OSError:
                 continue
-        return ''
+        else:
+            return ''
+        return wwn_path[len(glob_str):]
 
     def get_sysfs_wwid(self, device_names):
         """Return the wwid from sysfs in any of devices in udev format."""
@@ -232,23 +262,29 @@ class LinuxSCSI(executor.Executor):
         """
         # No used path happens on failed attachs, when we don't care about
         # individual flushes.
-        # When Nova used a multipath we don't need to do individual flushes.
-        if not path_used or was_multipath:
+        if not path_used:
             return False
 
-        # We need to flush the single path that was used.
-        # For encrypted volumes the symlink has been replaced, so realpath
-        # won't return device under /dev but under /dev/disk/...
         path = os.path.realpath(path)
         path_used = os.path.realpath(path_used)
-        return path_used == path or '/dev' != os.path.split(path_used)[0]
 
-    def remove_connection(self, devices_names, is_multipath, force=False,
-                          exc=None, path_used=None, was_multipath=False):
+        # Need to flush this device if we used this specific path.  We check
+        # this before checking if it's multipath in case we don't detect it
+        # being multipath correctly (as in bug #1897787).
+        if path_used == path:
+            return True
+
+        # We flush individual path if Nova didn't use a multipath and we
+        # replaced the symlink to a real device with a link to the decrypted
+        # DM.  We know we replaced it because it doesn't link to /dev/XYZ,
+        # instead it maps to /dev/mapped/crypt-XYZ
+        return not was_multipath and '/dev' != os.path.split(path_used)[0]
+
+    def remove_connection(self, devices_names, force=False, exc=None,
+                          path_used=None, was_multipath=False):
         """Remove LUNs and multipath associated with devices names.
 
         :param devices_names: Iterable with real device names ('sda', 'sdb')
-        :param is_multipath: Whether this is a multipath connection or not
         :param force: Whether to forcefully disconnect even if flush fails.
         :param exc: ExceptionChainer where to add exceptions if forcing
         :param path_used: What path was used by Nova/Cinder for I/O
@@ -257,22 +293,32 @@ class LinuxSCSI(executor.Executor):
         """
         if not devices_names:
             return
-        multipath_name = None
         exc = exception.ExceptionChainer() if exc is None else exc
-        LOG.debug('Removing %(type)s devices %(devices)s',
-                  {'type': 'multipathed' if is_multipath else 'single pathed',
-                   'devices': ', '.join(devices_names)})
 
-        if is_multipath:
-            multipath_dm = self.find_sysfs_multipath_dm(devices_names)
-            multipath_name = multipath_dm and self.get_dm_name(multipath_dm)
-            if multipath_name:
-                with exc.context(force, 'Flushing %s failed', multipath_name):
-                    self.flush_multipath_device(multipath_name)
-                    multipath_name = None
+        multipath_dm = self.find_sysfs_multipath_dm(devices_names)
+        LOG.debug('Removing %(type)s devices %(devices)s',
+                  {'type': 'multipathed' if multipath_dm else 'single pathed',
+                   'devices': ', '.join(devices_names)})
+        multipath_name = multipath_dm and self.get_dm_name(multipath_dm)
+        if multipath_name:
+            with exc.context(force, 'Flushing %s failed', multipath_name):
+                self.flush_multipath_device(multipath_name)
+                multipath_name = None
+            multipath_running = True
+        else:
+            multipath_running = self.is_multipath_running(
+                enforce_multipath=False, root_helper=self._root_helper)
 
         for device_name in devices_names:
             dev_path = '/dev/' + device_name
+            if multipath_running:
+                # Recent multipathd doesn't remove path devices in time when
+                # it receives mutiple udev events in a short span, so here we
+                # tell multipathd to remove the path device immediately.
+                # Even if this step fails, later removing an iscsi device
+                # triggers a udev event and multipathd can remove the path
+                # device based on the udev event
+                self.multipath_del_path(dev_path)
             flush = self.requires_flush(dev_path, path_used, was_multipath)
             self.remove_scsi_device(dev_path, force, exc, flush)
 
@@ -531,7 +577,7 @@ class LinuxSCSI(executor.Executor):
                                     root_helper=self._root_helper)
         return out
 
-    def extend_volume(self, volume_paths):
+    def extend_volume(self, volume_paths, use_multipath=False):
         """Signal the SCSI subsystem to test for volume resize.
 
         This function tries to signal the local system's kernel
@@ -561,24 +607,26 @@ class LinuxSCSI(executor.Executor):
             LOG.debug("volume size after scsi device rescan %s", new_size)
 
         scsi_wwn = self.get_scsi_wwn(volume_paths[0])
-        mpath_device = self.find_multipath_device_path(scsi_wwn)
-        if mpath_device:
-            # Force a reconfigure so that resize works
-            self.multipath_reconfigure()
+        if use_multipath:
+            mpath_device = self.find_multipath_device_path(scsi_wwn)
+            if mpath_device:
+                # Force a reconfigure so that resize works
+                self.multipath_reconfigure()
 
-            size = self.get_device_size(mpath_device)
-            LOG.info("mpath(%(device)s) current size %(size)s",
-                     {'device': mpath_device, 'size': size})
-            result = self.multipath_resize_map(scsi_wwn)
-            if 'fail' in result:
-                LOG.error("Multipathd failed to update the size mapping of "
-                          "multipath device %(scsi_wwn)s volume %(volume)s",
-                          {'scsi_wwn': scsi_wwn, 'volume': volume_paths})
-                return None
+                size = self.get_device_size(mpath_device)
+                LOG.info("mpath(%(device)s) current size %(size)s",
+                         {'device': mpath_device, 'size': size})
+                result = self.multipath_resize_map(scsi_wwn)
+                if 'fail' in result:
+                    LOG.error("Multipathd failed to update the size mapping "
+                              "of multipath device %(scsi_wwn)s volume "
+                              "%(volume)s",
+                              {'scsi_wwn': scsi_wwn, 'volume': volume_paths})
+                    return None
 
-            new_size = self.get_device_size(mpath_device)
-            LOG.info("mpath(%(device)s) new size %(size)s",
-                     {'device': mpath_device, 'size': new_size})
+                new_size = self.get_device_size(mpath_device)
+                LOG.info("mpath(%(device)s) new size %(size)s",
+                         {'device': mpath_device, 'size': new_size})
 
         return new_size
 
@@ -679,3 +727,11 @@ class LinuxSCSI(executor.Executor):
                                        check_exit_code=False,
                                        root_helper=self._root_helper)
         return stdout.strip() == 'ok'
+
+    def multipath_del_path(self, realpath):
+        """Remove a path from multipathd for monitoring."""
+        stdout, stderr = self._execute('multipathd', 'del', 'path', realpath,
+                                       run_as_root=True, timeout=5,
+                                       check_exit_code=False,
+                                       root_helper=self._root_helper)
+        return stdout.strip() == 'ok'
