@@ -45,8 +45,8 @@ class ScaleIOConnectorTestCase(test_connector.ConnectorTestCase):
             'scaleIO_volume_id': self.vol['provider_id'],
             'serverPort': 443,
             'serverUsername': 'test',
-            'serverPassword': 'fake',
-            'serverToken': 'fake_token',
+            'config_group': 'test',
+            'failed_over': False,
             'iopsLimit': None,
             'bandwidthLimit': None
         }
@@ -83,6 +83,9 @@ class ScaleIOConnectorTestCase(test_connector.ConnectorTestCase):
                          return_value=["emc-vol-{}".format(self.vol['id'])])
 
         # Patch scaleio privileged calls
+        self.get_password_mock = self.mock_object(scaleio.priv_scaleio,
+                                                  'get_connector_password',
+                                                  return_value='fake_password')
         self.get_guid_mock = self.mock_object(scaleio.priv_scaleio, 'get_guid',
                                               return_value=self.fake_guid)
         self.rescan_vols_mock = self.mock_object(scaleio.priv_scaleio,
@@ -168,6 +171,27 @@ class ScaleIOConnectorTestCase(test_connector.ConnectorTestCase):
         self.connector.connect_volume(self.fake_connection_properties)
         self.get_guid_mock.assert_called_once_with(
             self.connector.GET_GUID_OP_CODE)
+        self.get_password_mock.assert_called_once()
+
+    def test_connect_volume_old_connection_properties(self):
+        """Successful connect to volume"""
+        connection_properties = {
+            'hostIP': test_connector.MY_IP,
+            'serverIP': test_connector.MY_IP,
+            'scaleIO_volname': self.vol['name'],
+            'scaleIO_volume_id': self.vol['provider_id'],
+            'serverPort': 443,
+            'serverUsername': 'test',
+            'serverPassword': 'fake',
+            'serverToken': 'fake_token',
+            'iopsLimit': None,
+            'bandwidthLimit': None
+        }
+
+        self.connector.connect_volume(connection_properties)
+        self.get_guid_mock.assert_called_once_with(
+            self.connector.GET_GUID_OP_CODE)
+        self.get_password_mock.assert_not_called()
 
     def test_connect_volume_without_volume_id(self):
         """Successful connect to volume without a Volume Id"""
@@ -295,3 +319,14 @@ class ScaleIOConnectorTestCase(test_connector.ConnectorTestCase):
                          mock_device_size.return_value)
         self.rescan_vols_mock.assert_called_once_with(
             self.connector.RESCAN_VOLS_OP_CODE)
+
+    def test_connection_properties_without_failed_over(self):
+        """Handle connection properties with 'failed_over' missing"""
+        connection_properties = dict(self.fake_connection_properties)
+        connection_properties.pop('failed_over')
+
+        self.connector.connect_volume(connection_properties)
+        self.get_password_mock.assert_called_once_with(
+            scaleio.CONNECTOR_CONF_PATH,
+            connection_properties['config_group'],
+            False)
