@@ -230,15 +230,20 @@ class LinuxSCSITestCase(base.TestCase):
               {'do_raise': True, 'force': True})
     @ddt.unpack
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
+                       return_value=True)
     @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
-    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.remove_scsi_device')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
     def test_remove_connection_multipath_complete(self, remove_mock, wait_mock,
                                                   find_dm_mock,
                                                   get_dm_name_mock,
                                                   flush_mp_mock,
+                                                  is_mp_running_mock,
+                                                  mp_del_path_mock,
                                                   remove_link_mock,
                                                   do_raise, force):
         if do_raise:
@@ -246,7 +251,6 @@ class LinuxSCSITestCase(base.TestCase):
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
         mp_name = self.linuxscsi.remove_connection(devices_names,
-                                                   is_multipath=True,
                                                    force=mock.sentinel.Force,
                                                    exc=exc)
         find_dm_mock.assert_called_once_with(devices_names)
@@ -254,6 +258,9 @@ class LinuxSCSITestCase(base.TestCase):
         flush_mp_mock.assert_called_once_with(get_dm_name_mock.return_value)
         self.assertEqual(get_dm_name_mock.return_value if do_raise else None,
                          mp_name)
+        is_mp_running_mock.assert_not_called()
+        mp_del_path_mock.assert_has_calls([
+            mock.call('/dev/sda'), mock.call('/dev/sdb')])
         remove_mock.assert_has_calls([
             mock.call('/dev/sda', mock.sentinel.Force, exc, False),
             mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
@@ -262,6 +269,46 @@ class LinuxSCSITestCase(base.TestCase):
         remove_link_mock.assert_called_once_with(devices_names)
 
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
+                       return_value=True)
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm',
+                       return_value=None)
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
+    def test_remove_connection_multipath_complete_no_dm(self, remove_mock,
+                                                        wait_mock,
+                                                        find_dm_mock,
+                                                        get_dm_name_mock,
+                                                        flush_mp_mock,
+                                                        is_mp_running_mock,
+                                                        mp_del_path_mock,
+                                                        remove_link_mock):
+        devices_names = ('sda', 'sdb')
+        exc = exception.ExceptionChainer()
+        mp_name = self.linuxscsi.remove_connection(devices_names,
+                                                   force=mock.sentinel.Force,
+                                                   exc=exc)
+        find_dm_mock.assert_called_once_with(devices_names)
+        get_dm_name_mock.assert_not_called()
+        flush_mp_mock.assert_not_called()
+        self.assertIsNone(mp_name)
+        is_mp_running_mock.assert_called_once()
+        mp_del_path_mock.assert_has_calls([
+            mock.call('/dev/sda'), mock.call('/dev/sdb')])
+        remove_mock.assert_has_calls([
+            mock.call('/dev/sda', mock.sentinel.Force, exc, False),
+            mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
+        wait_mock.assert_called_once_with(devices_names)
+        self.assertFalse(bool(exc))
+        remove_link_mock.assert_called_once_with(devices_names)
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
+                       return_value=True)
     @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device',
                        side_effect=Exception)
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
@@ -270,32 +317,51 @@ class LinuxSCSITestCase(base.TestCase):
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
     def test_remove_connection_multipath_fail(self, remove_mock, wait_mock,
                                               find_dm_mock, get_dm_name_mock,
-                                              flush_mp_mock, remove_link_mock):
+                                              flush_mp_mock,
+                                              is_mp_running_mock,
+                                              mp_del_path_mock,
+                                              remove_link_mock):
         flush_mp_mock.side_effect = exception.ExceptionChainer
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
         self.assertRaises(exception.ExceptionChainer,
                           self.linuxscsi.remove_connection,
-                          devices_names, is_multipath=True,
-                          force=False, exc=exc)
+                          devices_names, force=False, exc=exc)
         find_dm_mock.assert_called_once_with(devices_names)
         get_dm_name_mock.assert_called_once_with(find_dm_mock.return_value)
         flush_mp_mock.assert_called_once_with(get_dm_name_mock.return_value)
+        is_mp_running_mock.assert_not_called()
+        mp_del_path_mock.assert_not_called()
         remove_mock.assert_not_called()
         wait_mock.assert_not_called()
         remove_link_mock.assert_not_called()
         self.assertTrue(bool(exc))
 
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
+                       return_value=True)
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
-    def test_remove_connection_singlepath(self, remove_mock, wait_mock,
-                                          remove_link_mock):
+    def test_remove_connection_singlepath_no_path(self, remove_mock, wait_mock,
+                                                  find_dm_mock,
+                                                  is_mp_running_mock,
+                                                  mp_del_path_mock,
+                                                  remove_link_mock):
+        # Test remove connection when we didn't form a multipath and didn't
+        # even use any of the devices that were found.  This means that we
+        # don't flush any of the single paths when removing them.
+        find_dm_mock.return_value = None
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
-        self.linuxscsi.remove_connection(devices_names, is_multipath=False,
+        self.linuxscsi.remove_connection(devices_names,
                                          force=mock.sentinel.Force,
                                          exc=exc)
+        find_dm_mock.assert_called_once_with(devices_names)
+        is_mp_running_mock.assert_called_once()
+        mp_del_path_mock.assert_has_calls([
+            mock.call('/dev/sda'), mock.call('/dev/sdb')])
         remove_mock.assert_has_calls(
             [mock.call('/dev/sda', mock.sentinel.Force, exc, False),
              mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
@@ -303,19 +369,33 @@ class LinuxSCSITestCase(base.TestCase):
         remove_link_mock.assert_called_once_with(devices_names)
 
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
+                       return_value=False)
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
     def test_remove_connection_singlepath_used(self, remove_mock, wait_mock,
+                                               find_dm_mock,
+                                               is_mp_running_mock,
+                                               mp_del_path_mock,
                                                remove_link_mock):
+        # Test remove connection when we didn't form a multipath and just used
+        # one of the single paths that were found.  This means that we don't
+        # flush any of the single paths when removing them.
+        find_dm_mock.return_value = None
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
 
         # realpath was mocked on test setup
         with mock.patch('os.path.realpath', side_effect=self.realpath):
-            self.linuxscsi.remove_connection(devices_names, is_multipath=True,
+            self.linuxscsi.remove_connection(devices_names,
                                              force=mock.sentinel.Force,
                                              exc=exc, path_used='/dev/sdb',
                                              was_multipath=False)
+        find_dm_mock.assert_called_once_with(devices_names)
+        is_mp_running_mock.assert_called_once()
+        mp_del_path_mock.assert_not_called()
         remove_mock.assert_has_calls(
             [mock.call('/dev/sda', mock.sentinel.Force, exc, False),
              mock.call('/dev/sdb', mock.sentinel.Force, exc, True)])
@@ -863,6 +943,16 @@ loop0                                     0"""
                                     'id': '0',
                                     'lun': '0'})
 
+    @mock.patch('six.moves.builtins.open')
+    def test_get_sysfs_wwn_mpath(self, open_mock):
+        wwn = '3600d0230000000000e13955cc3757800'
+        cm_open = open_mock.return_value.__enter__.return_value
+        cm_open.read.return_value = 'mpath-' + wwn
+
+        res = self.linuxscsi.get_sysfs_wwn(mock.sentinel.device_names, 'dm-1')
+        open_mock.assert_called_once_with('/sys/block/dm-1/dm/uuid')
+        self.assertEqual(wwn, res)
+
     @mock.patch('glob.glob')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
     def test_get_sysfs_wwn_single_designator(self, get_wwid_mock, glob_mock):
@@ -874,25 +964,67 @@ loop0                                     0"""
         glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
         get_wwid_mock.assert_called_once_with(mock.sentinel.device_names)
 
+    @mock.patch('six.moves.builtins.open', side_effect=Exception)
+    @mock.patch('glob.glob')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
+    def test_get_sysfs_wwn_mpath_exc(self, get_wwid_mock, glob_mock,
+                                     open_mock):
+        glob_mock.return_value = ['/dev/disk/by-id/scsi-wwid1',
+                                  '/dev/disk/by-id/scsi-wwid2']
+        get_wwid_mock.return_value = 'wwid1'
+        res = self.linuxscsi.get_sysfs_wwn(mock.sentinel.device_names, 'dm-1')
+        open_mock.assert_called_once_with('/sys/block/dm-1/dm/uuid')
+        self.assertEqual('wwid1', res)
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        get_wwid_mock.assert_called_once_with(mock.sentinel.device_names)
+
+    @mock.patch('os.listdir', return_value=['sda', 'sdd'])
     @mock.patch('os.path.realpath', side_effect=('/other/path',
+                                                 '/dev/dm-5',
                                                  '/dev/sda', '/dev/sdb'))
-    @mock.patch('os.path.islink', side_effect=(False, True, True, True, True))
-    @mock.patch('os.stat', side_effect=(False, True, True, True))
+    @mock.patch('os.path.islink', side_effect=(False,) + (True,) * 5)
+    @mock.patch('os.stat', side_effect=(False,) + (True,) * 4)
     @mock.patch('glob.glob')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
     def test_get_sysfs_wwn_multiple_designators(self, get_wwid_mock, glob_mock,
                                                 stat_mock, islink_mock,
-                                                realpath_mock):
+                                                realpath_mock, listdir_mock):
         glob_mock.return_value = ['/dev/disk/by-id/scsi-fail-link',
                                   '/dev/disk/by-id/scsi-fail-stat',
                                   '/dev/disk/by-id/scsi-non-dev',
+                                  '/dev/disk/by-id/scsi-another-dm',
                                   '/dev/disk/by-id/scsi-wwid1',
                                   '/dev/disk/by-id/scsi-wwid2']
+
         get_wwid_mock.return_value = 'pre-wwid'
         devices = ['sdb', 'sdc']
         res = self.linuxscsi.get_sysfs_wwn(devices)
         self.assertEqual('wwid2', res)
         glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        listdir_mock.assert_called_once_with('/sys/class/block/dm-5/slaves')
+        get_wwid_mock.assert_called_once_with(devices)
+
+    @mock.patch('os.listdir', side_effect=[['sda', 'sdb'], ['sdc', 'sdd']])
+    @mock.patch('os.path.realpath', side_effect=('/dev/sde',
+                                                 '/dev/dm-5',
+                                                 '/dev/dm-6'))
+    @mock.patch('os.path.islink', mock.Mock())
+    @mock.patch('os.stat', mock.Mock())
+    @mock.patch('glob.glob')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid', return_value='')
+    def test_get_sysfs_wwn_dm_link(self, get_wwid_mock, glob_mock,
+                                   realpath_mock, listdir_mock):
+        glob_mock.return_value = ['/dev/disk/by-id/scsi-wwid1',
+                                  '/dev/disk/by-id/scsi-another-dm',
+                                  '/dev/disk/by-id/scsi-our-dm']
+
+        devices = ['sdc', 'sdd']
+        res = self.linuxscsi.get_sysfs_wwn(devices)
+        self.assertEqual('our-dm', res)
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        listdir_mock.assert_has_calls(
+            [mock.call('/sys/class/block/dm-5/slaves'),
+             mock.call('/sys/class/block/dm-6/slaves')])
         get_wwid_mock.assert_called_once_with(devices)
 
     @mock.patch('os.path.realpath', side_effect=('/dev/sda', '/dev/sdb'))
@@ -911,6 +1043,16 @@ loop0                                     0"""
         glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
         get_wwid_mock.assert_called_once_with(devices)
 
+    @mock.patch('glob.glob', return_value=[])
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_sysfs_wwid')
+    def test_get_sysfs_wwn_no_links(self, get_wwid_mock, glob_mock):
+        get_wwid_mock.return_value = ''
+        devices = ['sdc']
+        res = self.linuxscsi.get_sysfs_wwn(devices)
+        self.assertEqual('', res)
+        glob_mock.assert_called_once_with('/dev/disk/by-id/scsi-*')
+        get_wwid_mock.assert_called_once_with(devices)
+
     @ddt.data({'wwn_type': 't10.', 'num_val': '1'},
               {'wwn_type': 'eui.', 'num_val': '2'},
               {'wwn_type': 'naa.', 'num_val': '3'})
@@ -1049,6 +1191,10 @@ loop0                                     0"""
         self.linuxscsi.multipath_add_path('/dev/sda')
         self.assertEqual(['multipathd add path /dev/sda'], self.cmds)
 
+    def test_multipath_del_path(self):
+        self.linuxscsi.multipath_del_path('/dev/sda')
+        self.assertEqual(['multipathd del path /dev/sda'], self.cmds)
+
     @ddt.data({'con_props': {}, 'dev_info': {'path': mock.sentinel.path}},
               {'con_props': None, 'dev_info': {'path': mock.sentinel.path}},
               {'con_props': {'device_path': mock.sentinel.device_path},
@@ -1079,8 +1225,15 @@ loop0                                     0"""
     def test_get_dev_path_no_path(self, con_props, dev_info):
         self.assertEqual('', self.linuxscsi.get_dev_path(con_props, dev_info))
 
-    @ddt.data(('/dev/sda', '/dev/sda', True, False, None),
+    @ddt.data(('/dev/sda', '/dev/sda', False, True, None),
+              # This checks that we ignore the was_multipath parameter if it
+              # doesn't make sense (because the used path is the one we are
+              # asking about)
+              ('/dev/sda', '/dev/sda', True, True, None),
               ('/dev/sda', '', True, False, None),
+              # Check for encrypted volume
+              ('/dev/link_sda', '/dev/disk/by-path/pci-XYZ', False, True,
+               ('/dev/sda', '/dev/mapper/crypt-pci-XYZ')),
               ('/dev/link_sda', '/dev/link_sdb', False, False, ('/dev/sda',
                                                                 '/dev/sdb')),
               ('/dev/link_sda', '/dev/link2_sda', False, True, ('/dev/sda',
